package madynes.flowoid;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.Scanner;

import madynes.flowoid.exporter.Driver;
import madynes.flowoid.exporter.ExporterService;
import madynes.flowoid.exporter.R;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.util.Log;

public class Connectivity extends BroadcastReceiver {
	private static Driver ipDriver;
	private static String ipInterfaceName;
	private static boolean enableDriverIP = false;
	private static boolean ipNetworkReachable;
	private static int statNetworkRestarts = 0;
	
	/**
	 * Defines and returns the network interface usable with IP.
	 * 
	 * @param context an Android context
	 * @return Network Interface
	 */
	public static String getIPInterface(Context context) {
		String name = null;
		
		// Find the interface associated to the default route
		try {
			Scanner routingTable = new Scanner(new FileInputStream("/proc/net/route"));
			
			// Header
			if(routingTable.hasNextLine()) {
				routingTable.nextLine();
			}
			
			// Routing lines
			while(routingTable.hasNextLine() && name == null) {
				String[] routingLine = routingTable.nextLine().split("\\s+");
				
				if(Integer.parseInt(routingLine[1], 16) == 0) {
					name = routingLine[0];
				}
			}
		} catch (FileNotFoundException e) {}

		// Alternate method: find the first active and not loopback network interface 
		if(name == null) {
			Log.d("DEBUG", "IP interface detected using the alternate method.");
			
			try {
				Enumeration<NetworkInterface> interfaces = NetworkInterface.getNetworkInterfaces();
				
				while(interfaces.hasMoreElements() && name == null) {
					NetworkInterface nInterface = interfaces.nextElement();
					
					if(nInterface.isUp() && !nInterface.isLoopback()) {
						name = nInterface.getName();
					}
				}
			} catch (SocketException e) {
				e.printStackTrace();
			}
		}
		
		return name;
	}
	
	/**
	 * Returns the network interface usable with IP. 
	 * 
	 * @return Network Interface
	 */
	public static String getIPInterfaceName() {
		return ipInterfaceName;
	}
	
	/**
	 * Starts IP driver.
	 * 
	 * @param context anAndroid context
	 */
	public static void startIPDriver(Context context) {
		enableDriverIP = true;
		ipInterfaceName = getIPInterface(context);

		if(ipInterfaceName != null) {
			String ipFilters = PreferencesActivity.getFiltersIP(context);
			ipDriver = Driver.use(context, R.raw.driver_ip, ipInterfaceName, ipFilters);

		} else {
			Log.d("EXPORTER", "Unable to find IP network interface");
		}
	}
	
	/**
	 * Stops IP driver.
	 */
	public static void stopIPDriver() {
		if(ipDriver != null) {
			ipDriver.stop();
		}
	}
	
	/**
	 * Returns true if IP Driver is enabled.
	 * 
	 * @return true if enabled
	 */
	public static boolean isIPDriverEnabled() {
		return enableDriverIP;
	}

	/**
	 * Disables IP Driver.
	 */
	public static void disableIPDriver() {
		enableDriverIP = false;
	}
	
	/**
	 * Returns the number of network state changes.
	 * 
	 * @return number of restarts
	 */
	public static int getStatNeworkRestarts() {
		return statNetworkRestarts;
	}

	/**
	 * Returns true if the device is (probably) currently Internet capable.
	 * 
	 * @return true if there is an active IP connection
	 */
	public static boolean isIPNetworkReachable() {
		return ipNetworkReachable;
	}
	
	@Override
	public void onReceive(Context context, Intent intent) {
		Log.d("DEBUG", "Network State changed");
		ipNetworkReachable = !intent.getBooleanExtra(ConnectivityManager.EXTRA_NO_CONNECTIVITY, false);
	
		if(enableDriverIP) {
			stopIPDriver();
		
			if(ipNetworkReachable && ExporterService.isRunning()) {
				startIPDriver(context);
				statNetworkRestarts++;
			}
		}
	}
}