package madynes.flowoid;

import madynes.flowoid.exporter.ExporterService;
import madynes.flowoid.exporter.R;
import madynes.flowoid.exporterImpl.Exporter;
import madynes.flowoid.exporterImpl.FieldDeviceID;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

public class MainActivity extends Activity {
		
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		
		PreferencesActivity.configureExporter(this);
		infosUpdate();
	}

	/**
	 * Starts the Exporter Service and Drivers. 
	 */
	private void start() {
		Toast.makeText(getApplicationContext(), getString(R.string.startingProbe), Toast.LENGTH_SHORT).show();

    	LocationFinder.setMinLocationInterval(PreferencesActivity.getIntPreference(this, "locationInterval", 1200));
    	LocationFinder.setMinLocationDistance(PreferencesActivity.getIntPreference(this, "locationDistance", 100));
		
		PreferencesActivity.configureExporter(this);
		ExporterService.start(this, Exporter.class);
		
		if(PreferencesActivity.getBooleanPreference(this, "enableDriverIP", true) &&
				(PreferencesActivity.getBooleanPreference(this, "enableIPv4", true) ||
				PreferencesActivity.getBooleanPreference(this, "enableIPv6", true))) {

			Connectivity.startIPDriver(this);

		} else {
			Connectivity.disableIPDriver();
		}
		
		showNotification();
	}
	
	/**
	 * Stops the Exporter Service and Drivers.
	 */
	private void stop() {
		Toast.makeText(getApplicationContext(), getString(R.string.stoppingProbe), Toast.LENGTH_SHORT).show();
		
		Connectivity.stopIPDriver();
		Intent intentService = ExporterService.getIntent();
		
		if(intentService != null) {
			stopService(intentService);
		}
		
		hideNotification();
	}
	
	/**
	 * Shows a permanent notification in the tray.
	 */
	private void showNotification() {
		Notification notification;
		NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
		CharSequence text = getString(R.string.notifMsg);

		notification = new Notification(R.drawable.icon, text, System.currentTimeMillis());
		notification.flags = Notification.FLAG_NO_CLEAR;
		PendingIntent contentIntent = PendingIntent.getActivity(this, 0, new Intent(this, MainActivity.class), 0);
		notification.setLatestEventInfo(this, getString(R.string.notifTitle), text, contentIntent);
		notificationManager.notify(R.string.app_name, notification);
	}
	
	/**
	 * Hides tray notification.
	 */
	private void hideNotification() {
		NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
		notificationManager.cancel(R.string.app_name);
	}

	/**
	 * Shows informations on the main screen.
	 */
	private void displayInfos() {
		ToggleButton switchButton = (ToggleButton) findViewById(R.id.switchButton);
		TextView mainStats1 = (TextView) findViewById(R.id.mainStats1);
		TextView mainStats2 = (TextView) findViewById(R.id.mainStats2);
		TextView mainStats3 = (TextView) findViewById(R.id.mainStats3);
		TextView mainStats4 = (TextView) findViewById(R.id.mainStats4);
		TextView mainInterface = (TextView) findViewById(R.id.mainInterface);
		TextView mainLocation = (TextView) findViewById(R.id.mainLocation);
		TextView mainDeviceID = (TextView) findViewById(R.id.mainDeviceID);
		TextView progressActiveNetflowsLabel = (TextView) findViewById(R.id.progressActiveNetflowsLabel);
		ProgressBar progressActiveNetflows = (ProgressBar) findViewById(R.id.progressActiveNetflows);

		String ipInterfaceName = Connectivity.getIPInterfaceName();
		String providerName = LocationFinder.getProviderName();
		String lastLocationLat = String.valueOf(LocationFinder.getInstance(this).getLatitude());
		String lastLocationLong = String.valueOf(LocationFinder.getInstance(this).getLongitude());
		int connectedDrivers = ExporterService.getStatConnectedDrivers();
		
		if(ExporterService.isRunning()) {
			switchButton.setChecked(true);
		}
		
		mainStats1.setText(getString(R.string.connectedDrivers) + " " + connectedDrivers);
		mainStats2.setText(getString(R.string.activeNetflows) + " " + ExporterService.getStatActiveNetflows());
		mainStats3.setText(getString(R.string.exportQueue) + " " + ExporterService.getStatNetflowsWaitingToBeExported());
		mainStats4.setText(getString(R.string.exportedNetflows) + " " + ExporterService.getStatExportedNetFlows());
		mainInterface.setText(getString(R.string.ipNetworkInterface) + " " + (ipInterfaceName == null ? getString(R.string.none) : ipInterfaceName));
		mainLocation.setText(Html.fromHtml(getString(R.string.locationProvider) + " " + (providerName == null ? getString(R.string.none) : "<a href='https://maps.google.com/maps?q=" + lastLocationLat + "," + lastLocationLong + "'>" + providerName + "</a>")));
		mainDeviceID.setText(Html.fromHtml(getString(R.string.deviceID) + " <strong>" + FieldDeviceID.getDeviceID(getContentResolver()) + "</strong>"));

		mainLocation.setMovementMethod(LinkMovementMethod.getInstance());
		
		if(ExporterService.isRunning() && connectedDrivers != 1) {
			mainStats1.setTextColor(connectedDrivers == 0 ? Color.RED : Color.BLUE);
		} else {
			mainStats1.setTextColor(mainDeviceID.getTextColors());
		}
		
		int currentNetflows = ExporterService.getStatActiveNetflows() + ExporterService.getStatNetflowsWaitingToBeExported();
		int maxNetflows = ExporterService.getPrefMaxNetflowsToTrack();
		
		progressActiveNetflowsLabel.setText(getString(R.string.maxNetflowsToTrack) + " (" + currentNetflows + "/" + maxNetflows + "):");
		progressActiveNetflows.setMax(maxNetflows);
		progressActiveNetflows.setProgress(currentNetflows);
		
		if(currentNetflows == maxNetflows) {
			progressActiveNetflowsLabel.setTextColor(Color.RED);
		} else {
			progressActiveNetflowsLabel.setTextColor(mainDeviceID.getTextColors());
		}
	}
	
	/**
	 * Updates regularly the displayed informations. 
	 */
	private void infosUpdate() {
		final Handler statsHandler = new Handler();
		
		Runnable statsRun = new Runnable() {
			public void run() {
				if(hasWindowFocus()) {
					displayInfos();
					statsHandler.postDelayed(this, 1000);
				}
			}
		};

		displayInfos();
		statsHandler.postDelayed(statsRun, 3000);
	}
	
	/**
	 * Starts or stops Flowoid processes on demand with the main button.
	 * 
	 * @param view current view
	 */
	public void switchButton(View view) {
		final ToggleButton button = (ToggleButton) findViewById(R.id.switchButton);
		button.setClickable(false);
		button.setEnabled(false);
		
		if(button.isChecked()) {
			start();
			button.setChecked(true);

		} else {
			stop();
			button.setChecked(false);
		}
		
		new Handler().postDelayed(new Runnable() {
			@Override
			public void run() {
				button.setClickable(true);
				button.setEnabled(true);
			}
		}, 5000);
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	        case R.id.menu_preferences:
	    		Intent intentPreferences = new Intent(this, PreferencesActivity.class);
	    		startActivity(intentPreferences);
	        	
	            return true;
	            
	        case R.id.menu_actions:
	    		Intent intentActions = new Intent(this, ActionsActivity.class);
	    		startActivity(intentActions);
	        	
	        	return true;
	            
	        case R.id.menu_stats:
	    		Intent intentStats = new Intent(this, StatsActivity.class);
	    		startActivity(intentStats);
	        	
	            return true;
	            
	        case R.id.menu_infos:
	        	AlertDialog.Builder builder = new AlertDialog.Builder(this);
	        	
	        	builder.setMessage(R.string.optionsInfosMsgUrl).setTitle(R.string.optionsInfosMsgTitle);
	        	builder.setPositiveButton(R.string.close, null);

	        	builder.create().show();
	        	
	            return true;

	        default:
	            return super.onOptionsItemSelected(item);
	    }
	}
	
	@Override
	public void onResume() {
		super.onResume();
		infosUpdate();
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.options, menu);

	    return true;
	}
}