#!/usr/bin/perl

use strict;
use warnings;
use Time::HiRes qw(gettimeofday tv_interval);

my ($optDebug, $optMinTopFreq, $optMinVariationSameness, $optMinCompressRate, $optMinCount, $optMinClusters, $optMaxGapSequence);

sub init {
	$optDebug = 0;
	$optMinTopFreq = 0;
	$optMinVariationSameness = .7;
	$optMinCompressRate = 0;
	$optMinCount = 0;
	$optMinClusters = 4;
	$optMaxGapSequence = 3600/3;
}

sub getStats {
	my ($name, $p_param, $start, $increment, $iterations, $arg) = @_;
	my ($cmd, $out, @outs, $t0, $t1, $duration);

	init();

	open STATFILE, ">>dvsm-stats-$name." . ($start * 100);
	open STATFILETIME, ">>dvsm-durations-$name." . ($start * 100);

	for($$p_param = $start; $$p_param < $start + $increment * $iterations; $$p_param += $increment) {
		$cmd = "./dvsm.pl --nodebug"
			. " --minTopFreq $optMinTopFreq"
			. " --minVariationSameness $optMinVariationSameness"
			. " --minCompressRate $optMinCompressRate"
			. " --minCount $optMinCount"
			. " --minClusters $optMinClusters"
			. " --maxGapSequence $optMaxGapSequence"
			. " $arg < dvsm.data";

		print "$cmd\n";

		$t0 = [gettimeofday];
		$out = `$cmd`;
		$t1 = [gettimeofday];

		chomp $out;
		@outs = split ' ', $out;
		$duration = tv_interval($t0, $t1);

		print STATFILE "$$p_param $outs[0]\n";
		print STATFILETIME "$$p_param $duration $outs[1]\n";
	}

	close STATFILE; 
	close STATFILETIME; 
}

#getStats "variations", \$optMinVariationSameness, $ARGV[0]*.13, .01, 13, "--statAvgVariations";
#getStats "top", \$optMinTopFreq, $ARGV[0]*10, 1, 10, "--statTopTokens";
#getStats "compress", \$optMinCompressRate, $ARGV[0]*.1, .01, 10, "--statAvgVariations"; # A REFAIRE AVEC --STATAVG
getStats "count", \$optMinCount, $ARGV[0]*.10, 1, 10, "--statAvgVariations"; # A REFAIRE AVEC 8..9

exit 0;
