#!/usr/bin/perl

use strict;
use warnings;
use diagnostics;
use Time::HiRes qw(gettimeofday tv_interval);
use DateTime::Format::DateParse;

my @netflows;

while (<STDIN>) {
        my ($startStr, $duration, $location) = $_ =~ /^([^ ]+\s+[^ ]+)\s+([^ ]+).*?((?:[0-9-]+\s+){3}\d+)\s+\S+\n$/;
	my $start = DateTime::Format::DateParse->parse_datetime($startStr)->epoch;
	my $end = DateTime::Format::DateParse->parse_datetime($startStr);
	$end->add(seconds => int($duration));

	$location =~ s/\s+/./;
	$location =~ s/\s+/,/;
	$location =~ s/\s+/./;
	push @netflows, [ $start, $location, $end ]; 
}

my @previous = ( 0, "", 0 );

print "LOCATION\t\tFROM\t\t\tTO\t\t\tCOORDINATES\n";

foreach (sort { @$a[0] <=> @$b[0] } @netflows) {
	my $realLocation = "Unknown";

	if(@$_[1] ne $previous[1]) {
		print $previous[2] . "\t$previous[1]\n" unless $previous[1] eq "";

		if(@$_[1] =~ /^(48.6495077,6.145881|48.6495673,6.1466816|48.6506423,6.1482039|48.6522343,6.149627|48.65427,6.151116|48.6688403,6.1621128|48.6688967,6.1621319|48.6689108,6.1621367|48.6689343,6.1621446|48.6703693,6.1611616|48.6714233,6.1647267|48.6767458,6.1587205)$/) {
			$realLocation = "Street\t";
		} elsif(@$_[1] =~ /^(48.6518384,6.1533198|48.6518401,6.1533172|48.6518405,6.1533167|48.6522332,6.1496204)$/) {
			$realLocation = "Work\t";
		}

		print "$realLocation\t\t" . DateTime->from_epoch(epoch => @$_[0])->set_time_zone("Europe/Paris") . "\t";
	}

	@previous = @$_;
}

print $previous[2] . "\t$previous[1]\n" unless $previous[1] eq "";

exit 0;
